@@ -89,6 +89,7 @@ warn = showIntAtBase 8 intToDigit ==> showOct
 -- LIST
 
 error = concat (map f x) ==> concatMap f x
+error = concat (fmap f x) ==> concatMap f x
 warn = concat [a, b] ==> a ++ b
 warn "Use map once" = map f (map g x) ==> map (f . g) x
 warn "Fuse concatMap/map" = concatMap f (map g x) ==> concatMap (f . g) x
@@ -283,15 +284,20 @@ warn "Redundant pair" = (fst x, snd x) ==>  x where note = DecreasesLaziness
 -- FUNCTOR
 
 error "Functor law" = fmap f (fmap g x) ==> fmap (f . g) x where _ = noQuickCheck
+error "Functor law" = f <$> g <$> x ==> f . g <$> x where _ = noQuickCheck
 error "Functor law" = fmap id ==> id where _ = noQuickCheck
+error "Functor law" = id <$> x ==> x where _ = noQuickCheck
 warn = fmap f $ x ==> f Control.Applicative.<$> x
     where _ = (isApp x || isAtom x) && noQuickCheck
 
 -- MONAD
 
 error "Monad law, left identity" = return a >>= f ==> f a where _ = noQuickCheck
+error "Monad law, left identity" = f =<< return a ==> f a where _ = noQuickCheck
 error "Monad law, right identity" = m >>= return ==> m where _ = noQuickCheck
+error "Monad law, right identity" = return =<< m ==> m where _ = noQuickCheck
 warn  = m >>= return . f ==> Control.Monad.liftM f m where _ = noQuickCheck -- cannot be fmap, because is in Functor not Monad
+warn  = return . f =<< m ==> Control.Monad.liftM f m where _ = noQuickCheck
 error = (if x then y else return ()) ==> Control.Monad.when x $ _noParen_ y where _ = not (isAtom y) && noQuickCheck
 error = (if x then y else return ()) ==> Control.Monad.when x y where _ = isAtom y && noQuickCheck
 error = (if x then return () else y) ==> Control.Monad.unless x $ _noParen_ y where _ = not (isAtom y) && noQuickCheck
@@ -304,11 +310,12 @@ warn  = flip forM ==> mapM where _ = noQuickCheck
 warn  = flip forM_ ==> mapM_ where _ = noQuickCheck
 error = when (not x) ==> unless x where _ = noQuickCheck
 error = x >>= id ==> Control.Monad.join x where _ = noQuickCheck
+error = id =<< x ==> Control.Monad.join x where _ = noQuickCheck
 error = liftM f (liftM g x) ==> liftM (f . g) x where _ = noQuickCheck
-error = fmap f (fmap g x) ==> fmap (f . g) x where _ = noQuickCheck
 warn  = a >> return () ==> Control.Monad.void a
     where _ = (isAtom a || isApp a) && noQuickCheck
 error = fmap (const ()) ==> Control.Monad.void where _ = noQuickCheck
+error = const () <$> x ==> Control.Monad.void x where _ = noQuickCheck
 error = flip (>=>) ==> (<=<) where _ = noQuickCheck
 error = flip (<=<) ==> (>=>) where _ = noQuickCheck
 error = flip (>>=) ==> (=<<) where _ = noQuickCheck
@@ -469,6 +476,7 @@ error "Evaluate" = False || x ==> x
 error "Evaluate" = not True ==> False
 error "Evaluate" = not False ==> True
 error "Evaluate" = Nothing >>= k ==> Nothing
+error "Evaluate" = k =<< Nothing ==> Nothing
 error "Evaluate" = either f g (Left x) ==> f x
 error "Evaluate" = either f g (Right y) ==> g y
 error "Evaluate" = fst (x,y) ==> x
@@ -530,6 +538,10 @@ yes = "test" ++ concatMap (' ':) ["of","this"] -- unwords ("test":["of","this"])
 yes = if f a then True else b -- f a || b
 yes = not (a == b) -- a /= b
 yes = not (a /= b) -- a == b
+yes = not . (a ==) -- (a /=)
+yes = not . (== a) -- (/= a)
+yes = not . (a /=) -- (a ==)
+yes = not . (/= a) -- (== a)
 yes = if a then 1 else if b then 1 else 2 -- if a || b then 1 else 2
 no  = if a then 1 else if b then 3 else 2
 yes = a >>= return . bob -- Control.Monad.liftM bob a
@@ -637,6 +649,7 @@ f condition tChar tBool = if condition then _monoField tChar else _monoField tBo
 foo = maybe Bar{..} id -- Data.Maybe.fromMaybe Bar{..}
 foo = (\a -> Foo {..}) 1
 foo = zipWith SymInfo [0 ..] (repeat ty) -- map (\ x -> SymInfo x ty) [0 ..]
+f rec = rec
 
 import Prelude \
 yes = flip mapM -- Control.Monad.forM
