@@ -1,4 +1,4 @@
-{-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE RecordWildCards, ViewPatterns #-}
 {-# OPTIONS_GHC -fno-warn-incomplete-patterns #-}
 
 module HLint(hlint, Suggestion, suggestionLocation, suggestionSeverity, Severity(..)) where
@@ -9,8 +9,15 @@ import System.Console.CmdArgs.Verbosity
 import Data.List
 import System.Exit
 import System.IO
+import System.IO.Extra
 import Prelude
 
+import Data.Version
+import System.Process.Extra
+import Data.Maybe
+import System.Directory
+import Text.ParserCombinators.ReadP
+
 import CmdLine
 import Settings
 import Report
@@ -23,7 +30,6 @@ import Util
 import Parallel
 import HSE.All
 
-
 -- | A suggestion - the @Show@ instance is of particular use.
 newtype Suggestion = Suggestion {fromSuggestion :: Idea}
                      deriving (Eq,Ord)
@@ -54,7 +60,7 @@ hlint :: [String] -> IO [Suggestion]
 hlint args = do
     cmd <- getCmd args
     case cmd of
-        CmdMain{} -> hlintMain cmd
+        CmdMain{} -> do xs <- hlintMain cmd; return $ if cmdNoExitCode cmd then [] else xs
         CmdGrep{} -> hlintGrep cmd >> return []
         CmdHSE{}  -> hlintHSE  cmd >> return []
         CmdTest{} -> hlintTest cmd >> return []
@@ -91,7 +97,7 @@ hlintGrep cmd@CmdGrep{..} = do
     if null cmdFiles then
         exitWithHelp
      else do
-        files <- concatMapM (resolveFile cmd) cmdFiles
+        files <- concatMapM (resolveFile cmd Nothing) cmdFiles
         if null files then
             error "No files found"
          else
@@ -102,16 +108,20 @@ hlintMain cmd@CmdMain{..} = do
     encoding <- if cmdUtf8 then return utf8 else readEncoding cmdEncoding
     let flags = parseFlagsSetExtensions (cmdExtensions cmd) $ defaultParseFlags{cppFlags=cmdCpp cmd, encoding=encoding}
     if null cmdFiles && not (null cmdFindHints) then do
-        hints <- concatMapM (resolveFile cmd) cmdFindHints
+        hints <- concatMapM (resolveFile cmd Nothing) cmdFindHints
         mapM_ (\x -> putStrLn . fst =<< findSettings2 flags x) hints >> return []
      else if null cmdFiles then
         exitWithHelp
-     else do
-        files <- concatMapM (resolveFile cmd) cmdFiles
-        if null files then
-            error "No files found"
-         else
-            runHints cmd{cmdFiles=files} flags
+     else if cmdRefactor then do
+         withTempFile (\t ->  runHlintMain cmd (Just t) flags)
+     else runHlintMain cmd Nothing flags
+
+runHlintMain :: Cmd -> Maybe FilePath -> ParseFlags -> IO [Suggestion]
+runHlintMain cmd@(CmdMain{..}) fp flags = do
+  files <- concatMapM (resolveFile cmd fp) cmdFiles
+  if null files
+    then error "No files found"
+    else runHints cmd{cmdFiles=files} flags
 
 readAllSettings :: Cmd -> ParseFlags -> IO [Setting]
 readAllSettings cmd@CmdMain{..} flags = do
@@ -131,11 +141,28 @@ runHints cmd@CmdMain{..} flags = do
         then applyHintFiles flags settings cmdFiles
         else concat <$> parallel [evaluateList =<< applyHintFile flags settings x Nothing | x <- cmdFiles]
     let (showideas,hideideas) = partition (\i -> cmdShowAll || ideaSeverity i /= Ignore) ideas
+    usecolour <- cmdUseColour cmd
+    showItem <- if usecolour then showANSI else return show
     if cmdJson
         then putStrLn . showIdeasJson $ showideas
+        else if cmdSerialise then do
+          hSetBuffering stdout NoBuffering
+          print $ map (\i -> (show i, ideaRefactoring i)) showideas
+        else if cmdRefactor then do
+          case cmdFiles of
+            [file] -> do
+              -- Ensure that we can find the executable
+              path <- checkRefactor (if cmdWithRefactor == "" then Nothing else Just cmdWithRefactor)
+              -- writeFile "hlint.refact"
+              let hints =  show $ map (\i -> (show i, ideaRefactoring i)) showideas
+              withTempFile $ \f -> do
+                writeFile f hints
+                runRefactoring path file f cmdRefactorOptions
+                -- Exit with the exit code from 'refactor'
+                >>= exitWith
+            _ -> error "Refactor flag can only be used with an individual file"
+
         else do
-            usecolour <- cmdUseColour cmd
-            showItem <- if usecolour then showANSI else return show
             mapM_ (outStrLn . showItem) showideas
             if null showideas then
                 when (cmdReports /= []) $ outStrLn "Skipping writing reports"
@@ -149,5 +176,29 @@ runHints cmd@CmdMain{..} flags = do
                     (let i = length hideideas in if i == 0 then "" else " (" ++ show i ++ " ignored)")
     return $ map Suggestion showideas
 
+runRefactoring :: FilePath -> FilePath -> FilePath -> String -> IO ExitCode
+runRefactoring rpath fin hints opts =  do
+  let args = [fin, "-v0"] ++ words opts ++ ["--refact-file", hints]
+  (_, _, _, phand) <- createProcess $ proc rpath args
+  hSetBuffering stdin LineBuffering
+  hSetBuffering stdout LineBuffering
+  -- Propagate the exit code from the spawn process
+  waitForProcess phand
+
+checkRefactor :: Maybe FilePath -> IO FilePath
+checkRefactor rpath = do
+  let excPath = (fromMaybe "refactor" rpath)
+  mexc <- findExecutable excPath
+  case mexc of
+    Just exc ->  do
+      vers <- readP_to_S parseVersion . tail <$> (readProcess exc ["--version"] "")
+      case vers of
+        [] -> putStrLn "Unabled to determine version of refactor" >> (return exc)
+        (last -> (version, _)) -> if versionBranch version >= [0,1,0,0]
+                                    then return exc
+                                    else error "Your version of refactor is too old, please upgrade to the latest version"
+    Nothing ->  error $ unlines ["Could not find refactor"
+                                , "Tried with: " ++ excPath ]
+
 evaluateList :: [a] -> IO [a]
 evaluateList xs = length xs `seq` return xs
