@@ -202,10 +202,12 @@ replaceBranches x = ([], \[] -> x)
 apps :: [Exp_] -> Exp_
 apps = foldl1 (App an)
 
-
 fromApps :: Exp_ -> [Exp_]
-fromApps (App _ x y) = fromApps x ++ [y]
-fromApps x = [x]
+fromApps = map fst . fromAppsWithLoc
+
+fromAppsWithLoc :: Exp_ -> [(Exp_, S)]
+fromAppsWithLoc (App l x y) = fromAppsWithLoc x ++ [(y, l)]
+fromAppsWithLoc x = [(x, ann x)]
 
 
 -- Rule for the Uniplate Apps functions
