@@ -35,6 +35,7 @@ import Data.Maybe
 import Data.Ord
 import Data.Either.Extra
 import Control.Monad
+import Refact.Types hiding (RType(Match))
 
 
 listRecHint :: DeclHint
@@ -46,7 +47,8 @@ listRecHint _ _ = concatMap f . universe
             (use,severity,x) <- matchListRec x
             let y = addCase x
             guard $ recursiveStr `notElem` varss y
-            return $ idea severity ("Use " ++ use) o y
+            -- Maybe we can do better here maintaining source formatting?
+            return $ (idea severity ("Use " ++ use) o y [Replace Decl (toSS o) [] (prettyPrint y)])
 
 
 recursiveStr = "_recursive_"
@@ -74,7 +76,7 @@ data Branch = Branch String [String] Int BList Exp_
 
 matchListRec :: ListCase -> Maybe (String,Severity,Exp_)
 matchListRec o@(ListCase vs nil (x,xs,cons))
-    
+
     | [] <- vs, nil ~= "[]", InfixApp _ lhs c rhs <- cons, opExp c ~= ":"
     , fromParen rhs =~= recursive, xs `notElem` vars lhs
     = Just $ (,,) "map" Error $ appsBracket
