@@ -1,14 +1,12 @@
-{-# LANGUAGE PatternGuards, ScopedTypeVariables, RecordWildCards, ViewPatterns #-}
+{-# LANGUAGE RecordWildCards #-}
 
 module Test.Util(
-	withTests, tested, passed, failed, progress
+    withTests, tested, passed, failed, progress
     ) where
 
 import Data.IORef
 import System.IO.Unsafe
-import Control.Exception
 import Control.Monad
-import System.IO
 
 
 data Result = Result {failures :: Int, total :: Int} deriving Show
@@ -17,11 +15,11 @@ data Result = Result {failures :: Int, total :: Int} deriving Show
 ref :: IORef [Result]
 ref = unsafePerformIO $ newIORef []
 
+
 -- | Returns the number of failing tests.
 --   Warning: Not multithread safe, but is reenterant
 withTests :: IO () -> IO Int
-withTests act = bracket (hGetBuffering stdout) (hSetBuffering stdout) $ const $ do
-    hSetBuffering stdout NoBuffering
+withTests act = do
     atomicModifyIORef ref $ \r -> (Result 0 0 : r, ())
     act
     Result{..} <- atomicModifyIORef ref $ \(r:rs) -> (rs, r)
@@ -36,12 +34,12 @@ progress = putChar '.'
 
 passed :: IO ()
 passed = do
-	atomicModifyIORef ref $ \(r:rs) -> (r{total=total r+1}:rs, ())
+    atomicModifyIORef ref $ \(r:rs) -> (r{total=total r+1}:rs, ())
 
 failed :: [String] -> IO ()
 failed xs = do
-	unless (null xs) $ putStrLn $ unlines $ "" : xs
-	atomicModifyIORef ref $ \(r:rs) -> (r{total=total r+1, failures=failures r+1}:rs, ())
+    unless (null xs) $ putStrLn $ unlines $ "" : xs
+    atomicModifyIORef ref $ \(r:rs) -> (r{total=total r+1, failures=failures r+1}:rs, ())
 
 tested :: Bool -> IO ()
 tested b = if b then passed else failed []
